=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2008-06-25 20:46:31 +0000
+++ bzrlib/tests/test_transform.py	2008-06-26 15:55:10 +0000
@@ -2224,3 +2224,50 @@
         summary = preview.get_preview_tree().path_content_summary('path')
         self.assertEqual(4, len(summary))
         self.assertEqual('tree-reference', summary[0])
+
+    def test_annotate(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file', 'a\n')])
+        tree.add('file', 'file-id')
+        tree.commit('a', rev_id='one')
+        self.build_tree_contents([('tree/file', 'a\nb\n')])
+        preview = TransformPreview(tree)
+        self.addCleanup(preview.finalize)
+        file_trans_id = preview.trans_id_file_id('file-id')
+        preview.delete_contents(file_trans_id)
+        preview.create_file('a\nb\nc\n', file_trans_id)
+        preview_tree = preview.get_preview_tree()
+        expected = [
+            ('one', 'a\n'),
+            ('me:', 'b\n'),
+            ('me:', 'c\n'),
+        ]
+        annotation = preview_tree.annotate_iter('file-id', 'me:')
+        self.assertEqual(expected, annotation)
+
+    def test_annotate_missing(self):
+        preview = self.get_empty_preview()
+        preview.new_file('file', preview.root, 'a\nb\nc\n', 'file-id')
+        preview_tree = preview.get_preview_tree()
+        expected = [
+            ('me:', 'a\n'),
+            ('me:', 'b\n'),
+            ('me:', 'c\n'),
+         ]
+        annotation = preview_tree.annotate_iter('file-id', 'me:')
+        self.assertEqual(expected, annotation)
+
+    def test_annotate_deleted(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file', 'a\n')])
+        tree.add('file', 'file-id')
+        tree.commit('a', rev_id='one')
+        self.build_tree_contents([('tree/file', 'a\nb\n')])
+        preview = TransformPreview(tree)
+        self.addCleanup(preview.finalize)
+        file_trans_id = preview.trans_id_file_id('file-id')
+        preview.delete_contents(file_trans_id)
+        preview_tree = preview.get_preview_tree()
+        annotation = preview_tree.annotate_iter('file-id', 'me:')
+        self.assertIs(None, annotation)
+

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2008-06-25 21:53:11 +0000
+++ bzrlib/transform.py	2008-06-26 15:55:10 +0000
@@ -22,6 +22,7 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
+    annotate,
     bzrdir,
     delta,
     errors,
@@ -1680,8 +1681,20 @@
 
     def annotate_iter(self, file_id,
                       default_revision=_mod_revision.CURRENT_REVISION):
-        return self._transform._tree.annotate_iter(file_id,
-            default_revision=default_revision)
+        changes = self._changes(file_id)
+        changed_content, versioned, kind = changes[2], changes[3], changes[6]
+        if kind[1] is None:
+            return None
+        if kind[0] is None or not versioned[0]:
+            old_annotation = []
+        else:
+            old_annotation = self._transform._tree.annotate_iter(file_id,
+                default_revision=default_revision)
+        if not changed_content:
+            return old_annotation
+        return annotate.reannotate([old_annotation],
+                                   self.get_file(file_id).readlines(),
+                                   default_revision)
 
     def get_symlink_target(self, file_id):
         """See Tree.get_symlink_target"""

